#!/opt/VRTSperl/bin/perl -w
#------------------------------------------------------------------
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#
# $Header: $
#
# $Author: $
# $Date: $
# $Revision: $
#------------------------------------------------------------------

BEGIN {
  my $sVCSperl = undef;
  my $sVCSagntfwHome = "/opt/VRTS/.ACCLib";
  ( my $sDirName = $0 ) =~ s#(.*)/.*$#$1#;

  if ( $ENV { CLUSTER_HOME } ) { # VCS 5.0/VAD..
    $sVCSperl = "$ENV{CLUSTER_HOME}/lib";
  }
  else { # VCS..
    $sVCSperl = "/opt/VRTSperl/lib/site_perl";
  }

  unshift ( @INC, $sDirName,
                  "$sVCSagntfwHome/vcs",
                  "$sVCSagntfwHome/system",
                  "$sVCSagntfwHome/network",
                  $sVCSperl );
}
use strict;
use POSIX;
use Getopt::Std;
use Fcntl;
use File::Path qw ( mkpath );

use constant VCSLOGDIR => "/var/VRTSvcs/log";


#
# print "\nThis script will start OracleApps component where systemd is supported.
#-------------------------------------------
# Create the Log Dir, If not already present
#-------------------------------------------

my %options = ();
getopts("r:s:u:o:O:a:", \%options);


   my    $sCommand =  $options{s};
   my    $sCommandOpts = $options{o};
   my    $sOutFile = $options{O};
   my    $sCmdArgs = $options{a};
   my    @sCommandArgsList = split (' ', $sCmdArgs );
   my    $sUser = $options{u};
   my    $ResName = $options{r};
   my    $riCommandReturnCode = ();
   my    $sDecryptedPassword = ();

my $sLogDir = VCSLOGDIR . "/OracleApps";

if ( !-d $sLogDir ) {
  if ( ! mkpath ( $sLogDir, 0, 0700 ) ) {
    warn "Failed to make log directory [$sLogDir]: $!\n";
    exit 1;
  }
}
#--------------------------------------
# Open the log file for printing output
#--------------------------------------
my @lsOsDetails = uname ();
my $sTime = strftime '%Y/%m/%d %H:%M:%S', localtime;
my $sHeader = "\n\n# $sTime\n";

if (!-e "$sLogDir/$ResName.log" ){
$sHeader = "#
# Log Name:     $sLogDir/$ResName\.log
# System:       $lsOsDetails[1]
# SysInfo:      $lsOsDetails[0]:$lsOsDetails[1],$lsOsDetails[3],$lsOsDetails[2],$lsOsDetails[-1]
# Created:      $sTime
# \n\n";
$sHeader =~ s/SunOS/Solaris/g;

}
sysopen ( LOG, "$sLogDir/$ResName.log",
          O_WRONLY|O_SYNC|O_CREAT|O_APPEND, 0641 ) ||
die "Failed to create and open file  [$sLogDir/$ResName\.log]: $!\n";
#print "Lddog file is created at: $sLogDir/MQ_update.log\n";

print LOG $sHeader;
   
   my $sDecryptCmd = "export VCS_LOG_CATEGORY=OracleAppsLOG; export VCS_LOG_RESOURCE_NAME=$ResName; export VCS_LOG_AGENT_NAME=OracleApps; export VCS_LOG_SCRIPT_NAME=StartWithSystemd.pl; /opt/VRTSvcs/sbin/vcsdecrypt";
   foreach my $sPswd (@sCommandArgsList){
     ( $sDecryptedPassword ) = grep {                             # Get decrypted..
                          s/([^\w-])/\\$1/g || 1          # Escape non-words..
                        } readpipe ( "$sDecryptCmd $sPswd" );
     $sPswd = $sDecryptedPassword."\n";
   }

 
if ( ! -x $sCommand )
  {
    print LOG "\n[$sCommand] is not executable. Check file existence and permissions.";
    exit 1;
  }
  my $sScript = $sCommand;
  $sCommand = $sCommand . " " . $sCommandOpts ;
  my $iChildPid = open( CMD, "|-");

  $SIG{PIPE} = sub {
                      print LOG "\nBroken pipe for [$sCommand]";
                     exit 1;
                   };

  if ( $iChildPid )
  {
    #------------------
    # Parent process.
    #------------------
    eval
    {

      for (@sCommandArgsList)
      {
        #--------------------------------
        # Write the commands to the pipe
        #--------------------------------
        print CMD ;
      }
      close ( CMD );
      $$riCommandReturnCode = $? >> 8;
      if ( $$riCommandReturnCode == 0 || $$riCommandReturnCode == 2 )
      {
        #--------------------------------
        # Command execution was good.
        #--------------------------------
        print LOG "\n[$sCommand] Command executed successfully. Return code [$$riCommandReturnCode]";
      }
      else
      {
        #------------------------------------------
        # Command execution returned some code.
        #------------------------------------------
         print LOG "\nCommand [$sCommand] execution was not successful.[$$riCommandReturnCode]";
      }

    }; #end of eval
  }
  elsif ( 0 == $iChildPid )
  {
    #------------------
    # Child process.
    #------------------
    my ( $sName      ,
         $sPassword  ,
         $iUid       ,
         $iGid       ,
         $iQuota     ,
         $sComment   ,
         $sGcos      ,
         $sHome      ,
         $sUserShell ,
         $sExpire    ) = getpwnam($sUser);

    #------------------------------------------
    # SetGid and Setuid of the User attribute
    #------------------------------------------
    POSIX::setgid($iGid);

    #--------------
    # Get File Gid
    #--------------

    my $iFileGid = 0;

    $iFileGid = ( stat $sScript ) [5];

     print LOG "\nStart script Group owner is [$iFileGid].";
    #----------------------
    # Get members of $fGid
    #----------------------

    my ( $sFileGrpName, $lnGrpMembers,
         @lsGrpMembers, $iMemberUid, $sFileUser ) = ();

    $sFileGrpName = getgrgid ( $iFileGid );
    $lnGrpMembers = ( getgrnam $sFileGrpName ) [3];
    @lsGrpMembers = split (' ', $lnGrpMembers );

    print LOG "\nStart script File's Group members are [$lnGrpMembers].";

    #------------------------------
    # Set $iUid is member of $fGid
    #------------------------------
    foreach $sFileUser ( @lsGrpMembers ){
        $iMemberUid = getpwnam ( $sFileUser );
        if ( $iMemberUid && $iMemberUid == $iUid ){
           $) = "$iGid $iFileGid $)";
           print LOG "\nSet Gid to [$iGid].";
        }
    }
    print LOG "\nSet Uid to [$iMemberUid].";

    #----------------------------------------------
    # Dup the stdout and stderr of this child to
    # be exec'd to the OutFile
    #----------------------------------------------
    open STDOUT, '>', "$sOutFile" or
    die "Can't redirect STDOUT to $sOutFile: $! ";
    open STDERR, ">&STDOUT" or
    die "Can't redirect STDERR to $sOutFile: $!";

    POSIX::setuid($iUid);
    #--------------
    # Exec Command
    #--------------
    exec("$sCommand ") ||
    print LOG "\nCannot execute [$sCommand]";
    exit 1;
  }
  else
  {
    #------------------------
    # Cannot fork process !
    #------------------------
    print LOG "\nCannot fork [$sCommand]";
    exit 1;
  }

    exit 0;
